#! /usr/bin/perl

## address.pl
##
## Author: Eric Marsden   <emarsden@mail.dotcom.fr>
## Time-stamp: <1998-01-25 ecm>


## Ce filtre prend en entree une liste d'adresses separees
## par des lignes vides et sort un fichier Postscript contenant
## les adresses mis en page pour etre imprimes sur de papier
## a etiquettes.
##
## Les variables au debut du programme permettent de parametrer
## la sortie :
##    - iPageWidth donne la largeur de la page
##    - iPageHeight donne l'hauteur de la page
##    - iHorizLabels est le nombre d'etiquettes horizontalement
##    - iVertLabels est le nombre d'etiquettes verticalement
##    - iOffsetX est le decalage horizontal par rapport a chaque etiquette
##    - iOffsetY                 vertical
##
## La taille du papier au format A4 est de 595x842 points Postscript
## (voir le Postscript FAQ pour d'autres formats)
##
## usage:  address.pl < address.txt > address.ps


require 'assert.pl';

$iPageWidth = 595;		# Poscript points, for A4 paper
$iPageHeight = 842;		# idem
$iHorizMargin = 10;		# margin on left and right of page, in points
$iVertMargin = 10;		# margin on top and bottom of page
$iHorizLabels = 2;		# labels per column
$iVertLabels = 12;		# labels per row
$iOffsetX = 10;			# horizontal offset into label (in  points)
$iOffsetY = 10;			# vertical offset into label
$iFontSize = 12;		# font size in points
$theFont = "Times-Roman";       # must be a Postscript font


## read addresses from stdin. Addresses are a sequence of lines,
## and are separated by empty lines. The addresses are stored in
## a list.

$iAddresses = 0;
$address = "";
@addressList = ();
while( $line = <STDIN> ) {
    if( ($line =~ /^[ \t\n\r\f]+$/) ) {
	unless( $address eq "" ) {
	    push( @addressList, $address );
	    $address = "";
	    ++$iAddresses;
	}
    } else {
	$address .= $line;
    }
}

$iPages = 1 + int( $iAddresses/($iHorizLabels*$iVertLabels) );
&printPostscriptHeader( STDOUT, $iPages );
$iPages = 0;
&printLabels( STDOUT );
&printPostscriptTrailer( STDOUT );





sub printLabels {
    my( $FILE ) = @_;

    whileLoop: while( 1 ) {
	print "%%Page: $iPages ";
	++$iPages;
	print "$iPages\n";
	for( $i = 0; $i < $iHorizLabels; ++$i ) {
	    for( $j = 0; $j < $iVertLabels; ++$j ) {
		$x = $i / $iHorizLabels * ($iPageWidth - 2 * $iHorizMargin);
		$y = $j / $iVertLabels  * ($iPageHeight- 2 * $iVertMargin);
		$x += $iHorizMargin + $iOffsetX;
		$y += $iVertMargin  + $iOffsetY;
		$address = pop( @addressList );
		&printLabel( STDOUT, $x, $y, $address );
		
		last whileLoop if @addressList == ();
	    }
	}
    }
  done:
} # end printLabels


## print the address starting at coordinates (x,y) on FILE.
## We mirror y because Postscript coordinates increase going
## up the page.
sub printLabel {
    my( $FILE, $x, $y, $address ) = @_;
    my( $mirrorY );
    
    @addressLines = split( '\n', $address );
    foreach( @addressLines ) {
	$mirrorY = $iPageHeight - $y;
	&assert( $_ ne "" );
	if( length > 20 ) {
	   $scaleFactor = 1.0 - ((length $_) - 20)/40;
	   print $FILE "gsave\n";
	   print $FILE "$x $mirrorY moveto ";
	   print $FILE "$scaleFactor 1.0 scale\n";
	   print $FILE "($_) show\n";
	   print $FILE "grestore\n";
	} else {
	   print $FILE "$x $mirrorY moveto ";
	   print $FILE "($_) show\n";
	}
	$y += $iFontSize;
    }
} # end sub printLabel


sub printPostscriptHeader {
    my( $FILE, $iNumPages ) = @_;
    my( $nowString );

    $nowString = localtime;
   
    print $FILE <<EOF
%!PS-Adobe-2.0
%%Title: address
%%Creator: PSaddress v1.0
%%DocumentFonts: $theFont
%%Pages: $iNumPages
%%PageOrder: Ascend
%%Orientation: Portrait
%%PaperSize: A4
%%CreationDate: $nowString
%%For: $ENV{'USER'}
%%EndComments

/reencdict 12 dict def /ReEncode { reencdict begin
/newcodesandnames exch def /newfontname exch def /basefontname exch def
/basefontdict basefontname findfont def /newfont basefontdict maxlength dict def
basefontdict { exch dup /FID ne { dup /Encoding eq
{ exch dup length array copy newfont 3 1 roll put }
{ exch newfont 3 1 roll put } ifelse } { pop pop } ifelse } forall
newfont /FontName newfontname put newcodesandnames aload pop
128 1 255 { newfont /Encoding get exch /.notdef put } for
newcodesandnames length 2 idiv { newfont /Encoding get 3 1 roll put } repeat
newfontname newfont definefont pop end } def
/isovec [ 
8#200 /grave 8#201 /acute 8#202 /circumflex 8#203 /tilde
8#204 /macron 8#205 /breve 8#206 /dotaccent 8#207 /dieresis
8#210 /ring 8#211 /cedilla 8#212 /hungarumlaut 8#213 /ogonek 8#214 /caron
8#220 /dotlessi 8#240 /space 8#241 /exclamdown 8#242 /cent 8#243 /sterling
8#244 /currency 8#245 /yen 8#246 /brokenbar 8#247 /section 8#250 /dieresis
8#251 /copyright 8#252 /ordfeminine 8#253 /guillemotleft 8#254 /logicalnot
8#255 /endash 8#256 /registered 8#257 /macron 8#260 /degree 8#261 /plusminus
8#262 /twosuperior 8#263 /threesuperior 8#264 /acute 8#265 /mu 8#266 /paragraph
8#267 /periodcentered 8#270 /cedilla 8#271 /onesuperior 8#272 /ordmasculine
8#273 /guillemotright 8#274 /onequarter 8#275 /onehalf 
8#276 /threequarters 8#277 /questiondown 8#300 /Agrave 8#301 /Aacute
8#302 /Acircumflex 8#303 /Atilde 8#304 /Adieresis 8#305 /Aring
8#306 /AE 8#307 /Ccedilla 8#310 /Egrave 8#311 /Eacute
8#312 /Ecircumflex 8#313 /Edieresis 8#314 /Igrave 8#315 /Iacute
8#316 /Icircumflex 8#317 /Idieresis 8#320 /Eth 8#321 /Ntilde 8#322 /Ograve
8#323 /Oacute 8#324 /Ocircumflex 8#325 /Otilde 8#326 /Odieresis 8#327 /multiply
8#330 /Oslash 8#331 /Ugrave 8#332 /Uacute 8#333 /Ucircumflex
8#334 /Udieresis 8#335 /Yacute 8#336 /Thorn 8#337 /germandbls 8#340 /agrave
8#341 /aacute 8#342 /acircumflex 8#343 /atilde 8#344 /adieresis 8#345 /aring
8#346 /ae 8#347 /ccedilla 8#350 /egrave 8#351 /eacute
8#352 /ecircumflex 8#353 /edieresis 8#354 /igrave 8#355 /iacute
8#356 /icircumflex 8#357 /idieresis 8#360 /eth 8#361 /ntilde 8#362 /ograve
8#363 /oacute 8#364 /ocircumflex 8#365 /otilde 8#366 /odieresis 8#367 /divide
8#370 /oslash 8#371 /ugrave 8#372 /uacute 8#373 /ucircumflex
8#374 /udieresis 8#375 /yacute 8#376 /thorn 8#377 /ydieresis ] def

/$theFont /isoFont isovec ReEncode
/isoFont findfont $iFontSize scalefont setfont
%%EndProlog

EOF
    
} # end printPostscriptHeader


sub printPostscriptTrailer {
    my( $FILE ) = @_;

    print $FILE "showpage\n";
    print $FILE "%%Trailer\n";
    print $FILE "%%end\n";
    print $FILE "%%EOF\n";
}
	
       
